module Jekyll
  # Link posts by UUIDs
  class LinkedPosts
    FIELDS = %w[lang related].freeze

    attr_reader :site

    def initialize(site)
      @site = site
    end

    # Process all the documents from the site and replace the UUID for
    # the actual post.
    #
    # If it doesn't find anything it'll show a warning and replace the
    # value for nil.
    def read
      docs.each do |doc|
        fields.each do |field|
          next unless doc.data.key? field

          # XXX: this lacks... elegance
          doc.data[field] =
            case doc.data[field]
              when String then
                find doc.data[field]
              when Array then
                doc.data[field].map do |doc|
                  find doc
                end.compact
              when Hash
                doc.data[field].map do |f, uuid|
                  [f, find(uuid)]
                end.to_h
              else
                Jekyll.logger.warn "Couldn't link posts on #{doc.path}"
                nil
              end
            end
      end
    end

    private

    # Obtain docs from the whole site
    def docs
      @docs ||= site.collections.map(&:last).map(&:docs).flatten
    end

    # Search for UUIDs in these front matter fields.  You can add more
    # as an Array called 'fields' on _config.yml for the plugin
    # configuration:
    #
    # linked_fields:
    #   - related
    #   - lang
    def fields
      @fields ||= site.config.fetch('linked_fields', FIELDS).compact.uniq
    end

    # Find a post with the given UUID
    def find(uuid)
      r = docs.lazy.find do |doc|
        doc.data['uuid'] == uuid
      end

      Jekyll.logger.warn "#{uuid} couldn't be found" if r.nil?

      r
    end
  end
end
